pkgname <- "lidR"
source(file.path(R.home("share"), "R", "examples-header.R"))
options(warn = 1)
library('lidR')

base::assign(".oldSearch", base::search(), pos = 'CheckExEnv')
base::assign(".old_wd", base::getwd(), pos = 'CheckExEnv')
cleanEx()
nameEx("Gatziolis2019")
### * Gatziolis2019

flush(stderr()); flush(stdout())

### Name: Gatziolis2019
### Title: Sensor tracking algorithm
### Aliases: Gatziolis2019

### ** Examples

# A valid file properly populated
LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile)
flightlines <- track_sensor(las, Gatziolis2019())

plot(las@header)
plot(flightlines, add = TRUE)



cleanEx()
nameEx("LAD")
### * LAD

flush(stderr()); flush(stdout())

### Name: LAD
### Title: Leaf area density
### Aliases: LAD

### ** Examples

z <- c(rnorm(1e4, 25, 6), rgamma(1e3, 1, 8)*6, rgamma(5e2, 5,5)*10)
z <- z[z<45 & z>0]

lad <- LAD(z)

plot(lad, type="l", xlab="Elevation", ylab="Leaf area density")



cleanEx()
nameEx("LAS-class")
### * LAS-class

flush(stderr()); flush(stdout())

### Name: LAS-class
### Title: An S4 class to represent a .las or .laz file
### Aliases: LAS-class LAS

### ** Examples

# Read a las/laz file
LASfile <- system.file("extdata", "example.laz", package="rlas")
las <- readLAS(LASfile)
las

# Creation of a LAS object out of external data
data <- data.frame(X = runif(100, 0, 100),
                   Y = runif(100, 0, 100),
                   Z = runif(100, 0, 20))

# 'data' has many decimal digits
data

# Create a default header and quantize *by reference*
# the coordinates to fit with offset and scale factors
cloud <- LAS(data)

# 'data' has been updated and coordinates were quantized
data
cloud

# Be careful when providing a header the function assumes that
# it corresponds to the data and won't quantize the coordinates
data <- data.frame(X = runif(100, 0, 100),
                   Y = runif(100, 0, 100),
                   Z = runif(100, 0, 20))
header <- las@header

# This works but triggers warnings and creates an invalid LAS object
cloud <- LAS(data, header)

las_check(cloud)



cleanEx()
nameEx("LAScatalog-class")
### * LAScatalog-class

flush(stderr()); flush(stdout())

### Name: LAScatalog-class
### Title: An S4 class to represent a catalog of .las or .laz files
### Aliases: LAScatalog-class

### ** Examples

## Not run: 
##D # Build a catalog
##D ctg <- readLAScatalog("filder/to/las/files/")
##D 
##D # Set some options
##D opt_filter(ctg) <- "-keep_first"
##D 
##D # Summary gives a summary of how the catalog will be processed
##D summary(ctg)
##D 
##D # We can seamlessly use lidR functions
##D hmean <- grid_metrics(ctg, mean(Z), 20)
##D ttops <- tree_detection(ctg, lmf(5))
##D 
##D # For low memory config it is probably advisable not to load entire files
##D # and process chunks instead
##D opt_chunk_size(ctg) <- 500
##D 
##D # Sometimes the output is likely to be very large
##D # e.g. large coverage and small resolution
##D dtm <- grid_terrain(ctg, 1, tin())
##D 
##D # In that case it is advisable to write the output(s) to files
##D opt_output_files(ctg) <- "path/to/folder/DTM_chunk_{XLEFT}_{YBOTTOM}"
##D 
##D # Raster will be written to disk. The list of written files is returned
##D # or, in this specific case, a virtual raster mosaic.
##D dtm <- grid_terrain(ctg, 1, tin())
##D 
##D # When chunks are files the original names of the las files can be preserved
##D opt_chunk_size(ctg) <- 0
##D opt_output_files(ctg) <- "path/to/folder/DTM_{ORIGINALFILENAME}"
##D dtm <- grid_terrain(ctg, 1, tin())
##D 
##D # For some functions, files MUST be written to disk. Indeed, it is certain that R cannot
##D # handle the entire output.
##D opt_chunk_size(ctg) <- 0
##D opt_output_files(ctg) <- "path/to/folder/{ORIGINALFILENAME}_norm"
##D opt_laz_compression(ctg) <- TRUE
##D new_ctg <- normalize_height(ctg, tin())
##D 
##D # The user has access to the catalog engine through the function catalog_apply
##D output <- catalog_apply(ctg, FUN, ...)
## End(Not run)



cleanEx()
nameEx("Roussel2020")
### * Roussel2020

flush(stderr()); flush(stdout())

### Name: Roussel2020
### Title: Sensor tracking algorithm
### Aliases: Roussel2020

### ** Examples

# A valid file properly populated
LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile)

# pmin = 15 because it is an extremely tiny file
# strongly decimated to reduce its size. There are
# actually few multiple returns
flightlines <- track_sensor(las, Roussel2020(pmin = 15))

plot(las@header)
plot(flightlines, add = TRUE)



cleanEx()
nameEx("VCI")
### * VCI

flush(stderr()); flush(stdout())

### Name: VCI
### Title: Vertical Complexity Index
### Aliases: VCI

### ** Examples

z <- runif(10000, 0, 10)

VCI(z, by = 1, zmax = 20)

z <- abs(rnorm(10000, 10, 1))

# expected to be closer to 0.
VCI(z, by = 1, zmax = 20)



cleanEx()
nameEx("add_attribute")
### * add_attribute

flush(stderr()); flush(stdout())

### Name: add_attribute
### Title: Add attributes into a LAS object
### Aliases: add_attribute add_lasattribute add_lasattribute_manual
###   add_lasrgb remove_lasattribute

### ** Examples

LASfile <- system.file("extdata", "example.laz", package="rlas")
las <- readLAS(LASfile, select = "xyz")

print(las)
print(las@header)

x <- 1:30

las <- add_attribute(las, x, "mydata")
print(las)        # The las object has a new attribute called "mydata"
print(las@header) # But the header has not been updated. This new data will not be written

las <- add_lasattribute(las, x, "mydata2", "A new data")
print(las)        # The las object has a new attribute called "mydata2"
print(las@header) # The header has been updated. This new data will be written

# Optionally if the data is already in the LAS object you can update the header skipping the
# parameter x
las <- add_lasattribute(las, name = "mydata", desc = "Amplitude")
print(las@header)

# Remove an extra bytes attribute
las <- remove_lasattribute(las, "mydata2")
print(las)
print(las@header)

las <- remove_lasattribute(las, "mydata")
print(las)
print(las@header)



cleanEx()
nameEx("asprs")
### * asprs

flush(stderr()); flush(stdout())

### Name: asprs
### Title: ASPRS LAS Classification
### Aliases: asprs LASNONCLASSIFIED LASUNCLASSIFIED LASGROUND
###   LASLOWVEGETATION LASMEDIUMVEGETATION LASHIGHVEGETATION LASBUILDING
###   LASLOWPOINT LASKEYPOINT LASWATER LASRAIL LASROADSURFACE LASWIREGUARD
###   LASWIRECONDUCTOR LASTRANSMISSIONTOWER LASBRIGDE LASNOISE
### Keywords: datasets

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile)
las2 = filter_poi(las, Classification %in% c(LASGROUND, LASWATER))

print(LASGROUND)



cleanEx()
nameEx("catalog_apply")
### * catalog_apply

flush(stderr()); flush(stdout())

### Name: catalog_apply
### Title: LAScatalog processing engine
### Aliases: catalog_apply catalog_sapply

### ** Examples

# More examples might be avaible in the official lidR vignettes or
# on the github book <https://jean-romain.github.io/lidRbook/>

## =========================================================================
## Example 1: detect all the tree tops over an entire catalog
## (this is basically a reproduction of the existing lidR function 'tree_detection')
## =========================================================================

# 1. Build the user-defined function that analyzes each chunk of the catalog.
# The function's first argument is a LAScluster object. The other arguments can be freely
# choosen by the user.
my_tree_detection_method <- function(cluster, ws)
{
  # The cluster argument is a LAScluster object. The user does not need to know how it works.
  # readLAS will load the region of interest (chunk) with a buffer around it, taking advantage of
  # point cloud indexation if possible. The filter and select options are propagated automatically
  las <- readLAS(cluster)
  if (is.empty(las)) return(NULL)

  # Find the tree tops using a user-developed method (here simply a LMF).
  ttops <- find_trees(las, lmf(ws))

  # ttops is a SpatialPointsDataFrame that contains the tree tops in our region of interest
  # plus the trees tops in the buffered area. We need to remove the buffer otherwise we will get
  # some trees more than once.
  bbox  <- raster::extent(cluster)
  ttops <- raster::crop(ttops, bbox)

  return(ttops)
}

# 2. Build a project (here, a single file catalog for the purposes of this dummmy example).
LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
prj <- readLAScatalog(LASfile)
plot(prj)

# 3. Set some processing options.
# For this dummy example, the chunk size is 100 m and the buffer is 10 m
opt_chunk_buffer(prj) <- 10
opt_chunk_size(prj)   <- 100            # small because this is a dummy example.
opt_chunk_alignment(prj) <- c(-50, -35) # Align such as it creates 2 chunks only.
opt_select(prj)       <- "xyz"          # Read only the coordinates.
opt_filter(prj)       <- "-keep_first"  # Read only first returns.

# 4. Apply a user-defined function to take advantage of the internal engine
opt    <- list(need_buffer = TRUE,   # catalog_apply will throw an error if buffer = 0
               automerge   = TRUE)   # catalog_apply will merge the outputs into a single object
output <- catalog_apply(prj, my_tree_detection_method, ws = 5, .options = opt)

spplot(output)

## Not run: 
##D 
##D ## ===================================================
##D ## Example 2: compute a rumple index on surface points
##D ## ===================================================
##D 
##D rumple_index_surface = function(cluster, res)
##D {
##D   las = readLAS(cluster)
##D   if (is.empty(las)) return(NULL)
##D 
##D   las    <- filter_surfacepoints(las, 1)
##D   rumple <- grid_metrics(las, ~rumple_index(X,Y,Z), res)
##D   bbox   <- raster::extent(cluster)
##D   rumple <- raster::crop(rumple, bbox)
##D 
##D   return(rumple)
##D }
##D 
##D LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
##D prj <- readLAScatalog(LASfile)
##D 
##D opt_chunk_buffer(prj) <- 1
##D opt_chunk_size(prj)   <- 140     # small because this is a dummy example.
##D opt_select(prj)       <- "xyz"   # read only the coordinates.
##D 
##D opt     <- list(raster_alignment = 20, # catalog_apply will adjust the chunks if required
##D                 automerge = TRUE)      # catalog_apply will merge the outputs into a single raster
##D output  <- catalog_apply(prj, rumple_index_surface, res = 20, .options = opt)
##D 
##D plot(output, col = height.colors(50))
## End(Not run)



cleanEx()
nameEx("catalog_options_tools")
### * catalog_options_tools

flush(stderr()); flush(stdout())

### Name: catalog_options_tools
### Title: Get or set LAScatalog processing engine options
### Aliases: catalog_options_tools opt_chunk_buffer opt_chunk_buffer<-
###   opt_chunk_size opt_chunk_size<- opt_chunk_alignment
###   opt_chunk_alignment<- opt_progress opt_progress<- opt_stop_early
###   opt_stop_early<- opt_wall_to_wall opt_wall_to_wall<-
###   opt_independent_files opt_independent_files<- opt_output_files
###   opt_output_files<- opt_laz_compression opt_laz_compression<-
###   opt_merge opt_merge<- opt_select opt_select<- opt_filter opt_filter<-

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
ctg = readLAScatalog(LASfile)

plot(ctg, chunk_pattern = TRUE)

opt_chunk_size(ctg) <- 150
plot(ctg, chunk_pattern = TRUE)

opt_chunk_buffer(ctg) <- 10
plot(ctg, chunk_pattern = TRUE)

opt_chunk_alignment(ctg) <- c(270,250)
plot(ctg, chunk_pattern = TRUE)

summary(ctg)

opt_output_files(ctg) <- "/path/to/folder/templated_filename_{XBOTTOM}_{ID}"
summary(ctg)



cleanEx()
nameEx("catalog_retile")
### * catalog_retile

flush(stderr()); flush(stdout())

### Name: catalog_retile
### Title: Retile a LAScatalog
### Aliases: catalog_retile

### ** Examples

## Not run: 
##D ctg = readLAScatalog("path/to/catalog")
##D 
##D # Create a new set of .las files 500 x 500 wide in the folder
##D # path/to/new/catalog/ and iteratively named Forest_1.las, Forest_2.las
##D # Forest_3.las, and so on.
##D 
##D opt_chunk_buffer(ctg) <- 0
##D opt_chunk_size(ctg) <- 500
##D opt_output_files(ctg) <- "path/to/new/catalog/Forest_{ID}
##D newctg = catalog_retile(ctg)
##D 
##D # Create a new set of .las files equivalent to the original,
##D # but extended with a 50 m buffer in the folder path/to/new/catalog/
##D # and iteratively named named after the original files.
##D 
##D opt_chunk_buffer(ctg) <- 50
##D opt_chunk_size(ctg) <- 0
##D opt_output_files(ctg) <- "path/to/new/catalog/{ORIGINALFILENAME}_buffered
##D newctg = catalog_retile(ctg)
##D 
##D # Create a new set of compressed .laz file equivalent to the original, keeping only
##D # first returns above 2 m
##D 
##D opt_chunk_buffer(ctg) <- 0
##D opt_chunk_size(ctg) <- 0
##D opt_laz_compression(ctg) <- TRUE
##D opt_filter(ctg) <- "-keep_first -drop_z_below 2"
##D newctg = catalog_retile(ctg)
## End(Not run)



cleanEx()
nameEx("catalog_select")
### * catalog_select

flush(stderr()); flush(stdout())

### Name: catalog_select
### Title: Select LAS files manually from a LAScatalog
### Aliases: catalog_select

### ** Examples

## Not run: 
##D ctg = readLAScatalog("<Path to a folder containing a set of .las files>")
##D new_ctg = catalog_select(ctg)
## End(Not run)



cleanEx()
nameEx("classify_ground")
### * classify_ground

flush(stderr()); flush(stdout())

### Name: classify_ground
### Title: Classify points as 'ground'
### Aliases: classify_ground

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, select = "xyzrn", filter = "-inside 273450 5274350 273550 5274450")

# Using the Progressive Morphological Filter
# --------------------------------------

# (Parameters chosen mainly for speed)
ws  <- seq(3,12, 4)
th  <- seq(0.1, 1.5, length.out = length(ws))

las <- classify_ground(las, pmf(ws, th))
#plot(las, color = "Classification")

#' # Using the Cloth Simulation Filter
# --------------------------------------

# (Parameters chosen mainly for speed)
mycsf <- csf(TRUE, 1, 1, time_step = 1)
las <- classify_ground(las, mycsf)
#plot(las, color = "Classification")



cleanEx()
nameEx("classify_noise")
### * classify_noise

flush(stderr()); flush(stdout())

### Name: classify_noise
### Title: Classify points as 'noise'
### Aliases: classify_noise

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, filter = "-inside 273450 5274350 273550 5274450")

# Add 20 artificial outliers
set.seed(314)
id = round(runif(20, 0, npoints(las)))
set.seed(42)
err = runif(20, -50, 50)
las$Z[id] = las$Z[id] + err

# Using SOR
las <- classify_noise(las, sor(15,7))
#plot(las, color = "Classification")

# Using IVF
las <- classify_noise(las, ivf(5,2))

# Remove outliers using filter_poi()
las_denoise <- filter_poi(las, Classification != LASNOISE)



cleanEx()
nameEx("clip")
### * clip

flush(stderr()); flush(stdout())

### Name: clip
### Title: Clip points in regions of interest
### Aliases: clip clip_roi clip_rectangle clip_polygon clip_circle
###   clip_transect

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")

# Load the file and clip the region of interest
las = readLAS(LASfile, select = "xyz", filter = "-keep_first")
subset1 = clip_rectangle(las, 684850, 5017850, 684900, 5017900)

# Do not load the file(s), extract only the region of interest
# from a bigger dataset
ctg = readLAScatalog(LASfile, progress = FALSE, filter = "-keep_first")
subset2 = clip_rectangle(ctg, 684850, 5017850, 684900, 5017900)

# Extract all the polygons from a shapefile
f <- system.file("extdata", "lake_polygons_UTM17.shp", package = "lidR")
lakes <- sf::st_read(f, quiet = TRUE)
subset3 <- clip_roi(las, lakes)

# Extract the polygons for a catalog, write them in files named
# after the lake names, do not load anything in R
opt_output_files(ctg) <- paste0(tempfile(), "_{LAKENAME_1}")
new_ctg = clip_roi(ctg, lakes)
plot(new_ctg)

# Extract a transect
p1 <- c(684800, y = 5017800)
p2 <- c(684900, y = 5017900)
tr1 <- clip_transect(las, p1, p2, width = 4)

## Not run: 
##D plot(subset1)
##D plot(subset2)
##D plot(subset3)
##D 
##D plot(tr1, axis = TRUE, clear_artifacts = FALSE)
## End(Not run)



cleanEx()
nameEx("cloud_metrics")
### * cloud_metrics

flush(stderr()); flush(stdout())

### Name: cloud_metrics
### Title: Compute metrics for a cloud of points
### Aliases: cloud_metrics

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
lidar = readLAS(LASfile)

cloud_metrics(lidar, ~max(Z))
cloud_metrics(lidar, ~mean(Intensity))

# Define your own new metrics
myMetrics = function(z, i)
{
  metrics = list(
     zwimean = sum(z*i)/sum(i), # Mean elevation weighted by intensities
     zimean  = mean(z*i),       # Mean products of z by intensity
     zsqmean = sqrt(mean(z^2))  # Quadratic mean
   )

   return(metrics)
}

metrics = cloud_metrics(lidar, ~myMetrics(Z, Intensity))

# Predefined metrics
cloud_metrics(lidar, .stdmetrics)



cleanEx()
nameEx("csf")
### * csf

flush(stderr()); flush(stdout())

### Name: csf
### Title: Ground Segmentation Algorithm
### Aliases: csf

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, select = "xyzrn")

mycsf <- csf(TRUE, 1, 1, time_step = 1)
las <- classify_ground(las, mycsf)
#plot(las, color = "Classification")



cleanEx()
nameEx("dalponte2016")
### * dalponte2016

flush(stderr()); flush(stdout())

### Name: dalponte2016
### Title: Individual Tree Segmentation Algorithm
### Aliases: dalponte2016

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, select = "xyz", filter = "-drop_z_below 0")
col <- pastel.colors(200)

chm <- grid_canopy(las, 0.5, p2r(0.3))
ker <- matrix(1,3,3)
chm <- raster::focal(chm, w = ker, fun = mean, na.rm = TRUE)

ttops <- find_trees(chm, lmf(4, 2))
las   <- segment_trees(las, dalponte2016(chm, ttops))
#plot(las, color = "treeID", colorPalette = col)



cleanEx()
nameEx("decimate_points")
### * decimate_points

flush(stderr()); flush(stdout())

### Name: decimate_points
### Title: Decimate a LAS object
### Aliases: decimate_points

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile, select = "xyz")

# Select points randomly to reach an overall density of 1
thinned1 = decimate_points(las, random(1))
#plot(grid_density(las))
#plot(grid_density(thinned1))

# Select points randomly to reach an homogeneous density of 1
thinned2 = decimate_points(las, homogenize(1,5))
#plot(grid_density(thinned2))

# Select the highest point within each pixel of an overlayed grid
thinned3 = decimate_points(las, highest(5))
#plot(thinned3)



cleanEx()
nameEx("delineate_crowns")
### * delineate_crowns

flush(stderr()); flush(stdout())

### Name: delineate_crowns
### Title: Compute the hull of each tree.
### Aliases: delineate_crowns

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
poi = "-drop_z_below 0 -inside 481280 3812940 481320 3812980"
las = readLAS(LASfile, select = "xyz0", filter = poi)

# NOTE: This dataset is already segmented

#plot(las, color = "treeID", colorPalette = pastel.colors(200))

# Only the hulls
convex_hulls = delineate_crowns(las)
plot(convex_hulls)

# The hulls + some user-defined metrics
convex_hulls = delineate_crowns(las, func = ~list(Zmean = mean(Z)))
convex_hulls

# The bounding box
bbox_hulls = delineate_crowns(las, "bbox")
plot(bbox_hulls)

## Not run: 
##D # With concave hull (longer to compute)
##D concave_hulls = delineate_crowns(las, "concave")
##D plot(concave_hulls)
##D 
##D spplot(convex_hulls, "ZTOP")
##D spplot(convex_hulls, "Zmean")
## End(Not run)



cleanEx()
nameEx("dsmtin")
### * dsmtin

flush(stderr()); flush(stdout())

### Name: dsmtin
### Title: Digital Surface Model Algorithm
### Aliases: dsmtin

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile)
col <- height.colors(50)

# Basic triangulation and rasterization of first returns
chm <- grid_canopy(las, res = 1, dsmtin())
plot(chm, col = col)

## Not run: 
##D # Potentially complex concave subset of point cloud
##D x = c(481340, 481340, 481280, 481300, 481280, 481340)
##D y = c(3812940, 3813000, 3813000, 3812960, 3812940, 3812940)
##D las2 = clip_polygon(las,x,y)
##D plot(las2)
##D 
##D # Since the TIN interpolation is done within the convex hull of the point cloud
##D # dummy pixels are interpolated that are strictly correct according to the interpolation method
##D # used, but meaningless in our CHM
##D chm <- grid_canopy(las2, res = 0.5, dsmtin())
##D plot(chm, col = col)
##D 
##D # Use 'max_edge' to trim dummy triangles
##D chm = grid_canopy(las2, res = 0.5, dsmtin(max_edge = 3))
##D plot(chm, col = col)
## End(Not run)



cleanEx()
nameEx("entropy")
### * entropy

flush(stderr()); flush(stdout())

### Name: entropy
### Title: Normalized Shannon diversity index
### Aliases: entropy

### ** Examples

z <- runif(10000, 0, 10)

# expected to be close to 1. The highest diversity is given for a uniform distribution
entropy(z, by = 1)

 z <- runif(10000, 9, 10)

# Must be 0. The lowest diversity is given for a unique possibility
entropy(z, by = 1)

z <- abs(rnorm(10000, 10, 1))

# expected to be between 0 and 1.
entropy(z, by = 1)



cleanEx()
nameEx("filter_poi")
### * filter_poi

flush(stderr()); flush(stdout())

### Name: filter_poi
### Title: Filter points of interest with matching conditions
### Aliases: filter_poi

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
lidar = readLAS(LASfile)

# Select the first returns classified as ground
firstground = filter_poi(lidar, Classification == 2L & ReturnNumber == 1L)

# Multiple arguments are equivalent to &
firstground = filter_poi(lidar, Classification == 2L, ReturnNumber == 1L)

# Multiple criteria
first_or_ground = filter_poi(lidar, Classification == 2L | ReturnNumber == 1L)



cleanEx()
nameEx("filters")
### * filters

flush(stderr()); flush(stdout())

### Name: filters
### Title: Predefined point of interest filters
### Aliases: filters filter_first filter_firstlast filter_firstofmany
###   filter_ground filter_last filter_nth filter_single

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
lidar = readLAS(LASfile)

firstReturns  = filter_first(lidar)
groundReturns = filter_ground(lidar)



cleanEx()
nameEx("find_localmaxima")
### * find_localmaxima

flush(stderr()); flush(stdout())

### Name: find_localmaxima
### Title: Local Maximum Filter
### Aliases: find_localmaxima

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, select = "xyzi", filter = "-drop_z_below 0 -keep_random_fraction 0.5")

# Using a 20x5 rectangle with a 45 degrees angle.
# This won't find the tree properly in the general case
# but may find some oriented structure.
lm = find_localmaxima(las, c(20, 5, pi/4))
plot(lm)



cleanEx()
nameEx("find_trees")
### * find_trees

flush(stderr()); flush(stdout())

### Name: find_trees
### Title: Individual tree detection
### Aliases: find_trees

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, select = "xyz", filter = "-inside 481250 3812980 481300 3813030")

ttops <- find_trees(las, lmf(ws = 5))

#x = plot(las)
#add_treetops3d(x, ttops)



cleanEx()
nameEx("gap_fraction_profile")
### * gap_fraction_profile

flush(stderr()); flush(stdout())

### Name: gap_fraction_profile
### Title: Gap fraction profile
### Aliases: gap_fraction_profile

### ** Examples

z <- c(rnorm(1e4, 25, 6), rgamma(1e3, 1, 8)*6, rgamma(5e2, 5,5)*10)
z <- z[z<45 & z>0]

hist(z, n=50)

gapFraction = gap_fraction_profile(z)

plot(gapFraction, type="l", xlab="Elevation", ylab="Gap fraction")



cleanEx()
nameEx("grid_canopy")
### * grid_canopy

flush(stderr()); flush(stdout())

### Name: grid_canopy
### Title: Digital Surface Model
### Aliases: grid_canopy

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, filter = "-inside 481280 3812940 481330 3812990")
col <- height.colors(50)

# Points-to-raster algorithm with a resolution of 1 meter
chm <- grid_canopy(las, res = 1, p2r())
plot(chm, col = col)

# Points-to-raster algorithm with a resolution of 0.5 meters replacing each
# point by a 20-cm radius circle of 8 points
chm <- grid_canopy(las, res = 0.5, p2r(0.2))
plot(chm, col = col)

# Basic triangulation and rasterization of first returns
chm <- grid_canopy(las, res = 0.5, dsmtin())
plot(chm, col = col)

# Khosravipour et al. pitfree algorithm
chm <- grid_canopy(las, res = 0.5, pitfree(c(0,2,5,10,15), c(0, 1.5)))
plot(chm, col = col)



cleanEx()
nameEx("grid_density")
### * grid_density

flush(stderr()); flush(stdout())

### Name: grid_density
### Title: Map the pulse or point density
### Aliases: grid_density

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile,  filter = "-inside 684800 5017800 684900 5017900")

d <- grid_density(las, 5)
plot(d)

las <- retrieve_pulses(las)
d <- grid_density(las)
plot(d)



cleanEx()
nameEx("grid_metrics")
### * grid_metrics

flush(stderr()); flush(stdout())

### Name: grid_metrics
### Title: Area-Based Approach
### Aliases: grid_metrics

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile)
col = height.colors(50)

# === Using all points ===

# Mean height with 400 m^2 cells
metrics = grid_metrics(las, ~mean(Z), 20)
plot(metrics, col = col)

# Define your own new metrics
myMetrics = function(z, i) {
  metrics = list(
     zwimean = sum(z*i)/sum(i), # Mean elevation weighted by intensities
     zimean  = mean(z*i),       # Mean products of z by intensity
     zsqmean = sqrt(mean(z^2))) # Quadratic mean

   return(metrics)
}

metrics = grid_metrics(las, ~myMetrics(Z, Intensity))

plot(metrics, col = col)
#plot(metrics, "zwimean", col = col)
#plot(metrics, "zimean", col = col)

# === With point filters ===

# Compute using only some points: basic
first = filter_poi(las, ReturnNumber == 1)
metrics = grid_metrics(first, ~mean(Z), 20)

# Compute using only some points: optimized
# faster and uses less memory. No intermediate object
metrics = grid_metrics(las, ~mean(Z), 20, filter = ~ReturnNumber == 1)

# Compute using only some points: best
# ~50% faster and uses ~10x less memory
las = readLAS(LASfile, filter = "-keep_first")
metrics = grid_metrics(las, ~mean(Z), 20)



cleanEx()
nameEx("grid_terrain")
### * grid_terrain

flush(stderr()); flush(stdout())

### Name: grid_terrain
### Title: Digital Terrain Model
### Aliases: grid_terrain

### ** Examples


LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile, filter = "-inside 273450 5274350 273550 5274450")
#plot(las)

dtm1 = grid_terrain(las, algorithm = knnidw(k = 6L, p = 2))
dtm2 = grid_terrain(las, algorithm = tin())

## Not run: 
##D dtm3 = grid_terrain(las, algorithm = kriging(k = 10L))
##D 
##D plot(dtm1)
##D plot(dtm2)
##D plot(dtm3)
##D plot_dtm3d(dtm1)
##D plot_dtm3d(dtm2)
##D plot_dtm3d(dtm3)
## End(Not run)



cleanEx()
nameEx("hexbin_metrics")
### * hexbin_metrics

flush(stderr()); flush(stdout())

### Name: hexbin_metrics
### Title: Area-Based Approach in hexagonal cells.
### Aliases: hexbin_metrics

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
lidar = readLAS(LASfile,filter = "-keep_random_fraction 0.5")

col = grDevices::colorRampPalette(c("blue", "cyan2", "yellow", "red"))

# Maximum elevation with a resolution of 8 m
hm = hexbin_metrics(lidar, ~max(Z), 8)
hexbin::plot(hm, colramp = col, main = "Max Z")

# Mean height with a resolution of 20 m
hm = hexbin_metrics(lidar, ~mean(Z), 20)
hexbin::plot(hm, colramp = col, main = "Mean Z")

# Define your own new metrics
myMetrics = function(z, i)
{
  metrics = list(
    zwimean = sum(z*i)/sum(i), # Mean elevation weighted by intensities
    zimean  = mean(z*i),       # Mean products of z by intensity
  zsqmean = sqrt(mean(z^2))    # Quadratic mean
  )

  return(metrics)
}

metrics = hexbin_metrics(lidar, ~myMetrics(Z, Intensity), 10)

hexbin::plot(metrics$zwimean, colramp = col, main = "zwimean")
#hexbin::plot(metrics$zimean, colramp = col, main = "zimean")
#hexbin::plot(metrics$zsqmean, colramp = col, main = "zsqmean")



cleanEx()
nameEx("homogenize")
### * homogenize

flush(stderr()); flush(stdout())

### Name: homogenize
### Title: Point Cloud Decimation Algorithm
### Aliases: homogenize

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile, select = "xyz")

# Select points randomly to reach an homogeneous density of 1
thinned = decimate_points(las, homogenize(1,5))
plot(grid_density(thinned, 10))



cleanEx()
nameEx("identify")
### * identify

flush(stderr()); flush(stdout())

### Name: retrieve_pulses
### Title: Retrieve individual pulses, flightlines or scanlines
### Aliases: retrieve_pulses retrieve_flightlines retrieve_scanlines

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile)

las <- retrieve_pulses(las)
las

las <- retrieve_flightlines(las)
#plot(las, color = "flightlineID")



cleanEx()
nameEx("is")
### * is

flush(stderr()); flush(stdout())

### Name: is
### Title: A set of boolean tests on objects
### Aliases: is is.empty is.overlapping is.indexed is.algorithm
###   is.parallelised

### ** Examples

LASfile <- system.file("extdata", "example.laz", package="rlas")
las = readLAS(LASfile)
is.empty(las)

las = new("LAS")
is.empty(las)

f <- lmf(2)
is.parallelised(f)

g <- pitfree()
is.parallelised(g)

ctg <- readLAScatalog(LASfile)
is.indexed(ctg)



cleanEx()
nameEx("ivf")
### * ivf

flush(stderr()); flush(stdout())

### Name: ivf
### Title: Noise Segmentation Algorithm
### Aliases: ivf

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, filter = "-inside 273450 5274350 273550 5274450")

# Add some artificial outliers
set.seed(314)
id = round(runif(20, 0, npoints(las)))
set.seed(42)
err = runif(20, -50, 50)
las$Z[id] = las$Z[id] + err

las <- classify_noise(las, ivf(5,2))



cleanEx()
nameEx("knnidw")
### * knnidw

flush(stderr()); flush(stdout())

### Name: knnidw
### Title: Spatial Interpolation Algorithm
### Aliases: knnidw

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile)

#plot(las)

dtm = grid_terrain(las, algorithm = knnidw(k = 6L, p = 2))

#plot(dtm, col = terrain.colors(50))
#plot_dtm3d(dtm)



cleanEx()
nameEx("kriging")
### * kriging

flush(stderr()); flush(stdout())

### Name: kriging
### Title: Spatial Interpolation Algorithm
### Aliases: kriging

### ** Examples

## Not run: 
##D LASfile <- system.file("extdata", "Topography.laz", package="lidR")
##D las = readLAS(LASfile)
##D 
##D # plot(las)
##D 
##D dtm = grid_terrain(las, algorithm = kriging())
##D 
##D plot(dtm, col = terrain.colors(50))
##D plot_dtm3d(dtm)
## End(Not run)



cleanEx()
nameEx("las_check")
### * las_check

flush(stderr()); flush(stdout())

### Name: las_check
### Title: Inspect a LAS object
### Aliases: las_check

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile)
las_check(las)



cleanEx()
nameEx("las_utilities")
### * las_utilities

flush(stderr()); flush(stdout())

### Name: las_utilities
### Title: LAS utilities
### Aliases: las_utilities las_rescale las_reoffset las_quantize las_update
###   quantize is.quantized count_not_quantized storable_coordinate_range

### ** Examples

LASfile <- system.file("extdata", "example.laz", package="rlas")
las = readLAS(LASfile)

# Manual modification of the coordinates (e.g. rotation, re-alignment, ...)
las@data$X <- las@data$X + 2/3
las@data$Y <- las@data$Y - 5/3

# The point cloud is no longer valid
las_check(las)

# It is important to fix that
las_quantize(las)

# Now the file is almost valid
las_check(las)

# Update the object to set up-to-date header data
las <- las_update(las)
las_check(las)

# In practice the above code is not useful for regular users because the operators
# $<- already perform such operations on-the-fly. Thus the following
# syntax must be preferred and returns valid objects. Previous tools
# were only intended to be used in very specific cases.
las$X <- las$X + 2/3
las$Y <- las$Y - 5/3

# Rescale and reoffset recompute the coordinates with
# new scales and offsets according to LAS specification
las <- las_rescale(las, xscale = 0.01, yscale = 0.01)
las <- las_reoffset(las, xoffset = 300000, yoffset = 5248000)



cleanEx()
nameEx("li2012")
### * li2012

flush(stderr()); flush(stdout())

### Name: li2012
### Title: Individual Tree Segmentation Algorithm
### Aliases: li2012

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, select = "xyz", filter = "-drop_z_below 0")
col <- pastel.colors(200)

las <- segment_trees(las, li2012(dt1 = 1.4))
#plot(las, color = "treeID", colorPalette = col)



cleanEx()
nameEx("lidR-spatial-index")
### * lidR-spatial-index

flush(stderr()); flush(stdout())

### Name: lidR-spatial-index
### Title: Spatial index
### Aliases: lidR-spatial-index sensor sensor<- index index<-

### ** Examples

LASfile <- system.file("extdata", "example.laz", package="rlas")
las <- readLAS(LASfile)

# By default the sensor and spatial index codes are 0
sensor(las)
index(las)

# Codes are used internally and not intended to be known by users
# Use h option for human readable output
sensor(las, h = TRUE)
index(las, h = TRUE)

# Modification of the sensor enables users to select a better spatial index
# when processing the point-cloud.
sensor(las) <- "tls"
sensor(las, h = TRUE)
index(las, h = TRUE)

# Modification of the spatial index forces users to choose one of the available
# spatial indexes.
index(las) <- "quadtree"
sensor(las, h = TRUE)
index(las, h = TRUE)

# The simplest way to take advantage of appropriate spatial indexing is
# to use one of the read*LAS() functions.
las <- readTLSLAS(LASfile)
sensor(las, h = TRUE)
index(las, h = TRUE)

# But for some specific point-clouds / algorithms it might be advisable to force
# the use of a specific spatial index to perform the computation faster
index(las) <- "voxelpartition"
index(las, h = TRUE)

# With a LAScatalog, spatial indexing information is propagated to the
# different chunks
ctg = readTLSLAScatalog(LASfile)
index(ctg) <- "voxelpartition"
sensor(ctg, h = TRUE)
index(ctg, h = TRUE)

# ==================
# PERFORMANCE TESTS
# ==================

## Not run: 
##D # Performance tests on TLS
##D # ------------------------
##D 
##D # The package does not include TLS data
##D # so we can generate something that looks TLS-ish
##D # >>>>>>>>>>>
##D X = runif(50, -25, 25)
##D Y = runif(50, -25, 25)
##D X = as.numeric(sapply(X, function(x) rnorm(2000, x, 2)))
##D Y = as.numeric(sapply(Y, function(x) rnorm(2000, x, 2)))
##D Z = abs(rnorm(length(Y), 10, 5))
##D veg = data.frame(X,Y,Z)
##D X = runif(5000, -30, 30)
##D Y = runif(5000, -30, 30)
##D Z = runif(5000, 0, 1)
##D ground = data.frame(X,Y,Z)
##D X = runif(30, -30, 30)
##D Y = runif(30, -30, 30)
##D Z = runif(30, 0, 30)
##D noise = data.frame(X,Y,Z)
##D las = LAS(rbind(ground, veg, noise))
##D # <<<<<<<<<<<<<
##D 
##D plot(las)
##D 
##D # If read with readALSLAS()
##D sensor(las) <- "als"
##D system.time(classify_noise(las, sor(20, 8)))
##D #> 1.5 sec
##D 
##D # If read with readTLSLAS()
##D sensor(las) <- "tls"
##D system.time(classify_noise(las, sor(20, 8)))
##D #> 0.6 sec
##D 
##D # Performance tests on ALS
##D # ------------------------
##D 
##D # The package does not include large ALS data
##D # so we can generate something that looks ALS-ish
##D # >>>>>>>>>>>
##D X = runif(4e5, 0, 1000)
##D Y = runif(4e5, 0, 1000)
##D Z = 40*sin(0.01*X) + 50*cos(0.005*Y) + abs(rnorm(length(Y), 10, 5))
##D veg = data.frame(X,Y,Z)
##D X = runif(100, 0, 1000)
##D Y = runif(100, 0, 1000)
##D Z = 40*sin(0.01*X) + 50*cos(0.005*Y) + abs(rnorm(length(Y), 10, 5)) + runif(100, 30, 70)
##D noise = data.frame(X,Y,Z)
##D las = LAS(rbind(veg, noise))
##D # <<<<<<<<<<<<<
##D 
##D plot(las)
##D 
##D # If read with readALSLAS()
##D sensor(las) <- "als"
##D system.time(classify_noise(las, sor(15, 8)))
##D #> 3 sec
##D 
##D # If read with readTLSLAS()
##D sensor(las) <- "tls"
##D system.time(classify_noise(las, sor(15, 8)))
##D #> 4.3 sec
## End(Not run)



cleanEx()
nameEx("lmf")
### * lmf

flush(stderr()); flush(stdout())

### Name: lmf
### Title: Individual Tree Detection Algorithm
### Aliases: lmf

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, select = "xyz", filter = "-inside 481250 3812980 481300 3813050")

# point-cloud-based
# =================

# 5x5 m fixed window size
ttops <- find_trees(las, lmf(5))

#x <- plot(las)
#add_treetops3d(x, ttops)

# variable windows size
f <- function(x) { x * 0.07 + 3}
ttops <- find_trees(las, lmf(f))

#x <- plot(las)
#add_treetops3d(x, ttops)

# raster-based
# ============

chm <- grid_canopy(las, res = 1, p2r(0.15))
ttops <- find_trees(chm, lmf(5))

plot(chm, col = height.colors(30))
plot(ttops, add = TRUE)

# variable window size
f <- function(x) { x * 0.07 + 3 }
ttops <- find_trees(chm, lmf(f))

plot(chm, col = height.colors(30))
plot(ttops, add = TRUE)



cleanEx()
nameEx("manual")
### * manual

flush(stderr()); flush(stdout())

### Name: manual
### Title: Individual Tree Detection Algorithm
### Aliases: manual

### ** Examples

## Not run: 
##D LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
##D las = readLAS(LASfile)
##D 
##D # Full manual tree detection
##D ttops = find_trees(las, manual())
##D 
##D # Automatic detection with manual correction
##D ttops = find_trees(las, lmf(5))
##D ttops = find_trees(las, manual(ttops))
## End(Not run)



cleanEx()
nameEx("maxima")
### * maxima

flush(stderr()); flush(stdout())

### Name: maxima
### Title: Point Cloud Decimation Algorithm
### Aliases: maxima highest lowest

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile, select = "xyz")

# Select the highest point within each cell of an overlayed grid
thinned = decimate_points(las, highest(4))
#plot(thinned)

# Select the lowest point within each cell of an overlayed grid
thinned = decimate_points(las, lowest(4))
#plot(thinned)



cleanEx()
nameEx("merge_spatial")
### * merge_spatial

flush(stderr()); flush(stdout())

### Name: merge_spatial
### Title: Merge a point cloud with a source of spatial data
### Aliases: merge_spatial

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
shp     <- system.file("extdata", "lake_polygons_UTM17.shp", package = "lidR")

las   <- readLAS(LASfile, filter = "-keep_random_fraction 0.1")
lakes <- sf::st_read(shp)

# The attribute "inlake" does not exist in the shapefile.
# Points are classified as TRUE if in a polygon
las    <- merge_spatial(las, lakes, "inlakes")     # New attribute 'inlakes' is added.
forest <- filter_poi(las, inlakes == FALSE)
#plot(forest)

# The attribute "LAKENAME_1" exists in the shapefile.
# Points are classified with the values of the polygons
las <- merge_spatial(las, lakes, "LAKENAME_1")     # New column 'LAKENAME_1' is added.



cleanEx()
nameEx("normalize_height")
### * normalize_height

flush(stderr()); flush(stdout())

### Name: normalize_height
### Title: Remove the topography from a point cloud
### Aliases: normalize_height unnormalize_height -,LAS,RasterLayer-method
###   -,LAS,lidRAlgorithm-method

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, filter = "-inside 273450 5274350 273550 5274450")

#plot(las)

# First option: use a RasterLayer as DTM
# =======================================================

dtm <- grid_terrain(las, 1, knnidw(k = 6L, p = 2))
las <- normalize_height(las, dtm)

plot(dtm)
#plot(las)

# restore original elevations
las <- unnormalize_height(las)
#plot(las)

# operator - can be used. This is equivalent to the previous
las <- las - dtm
#plot(las)

# restore original elevations
las <- unnormalize_height(las)

# Second option: interpolate each point (no discretization)
# =========================================================

las <- normalize_height(las, tin())
#plot(las)

# operator - can be used. This is equivalent to the previous
las <- unnormalize_height(las)
las <- las - tin()

## Not run: 
##D # All the following syntaxes are correct
##D las <- normalize_height(las, knnidw())
##D las <- normalize_height(las, knnidw(k = 8, p = 2))
##D las <- las - knnidw()
##D las <- las - knnidw(k = 8)
##D las <- normalize_height(las, kriging())
##D las <- las - kriging(k = 8)
## End(Not run)




cleanEx()
nameEx("normalize_intensity")
### * normalize_intensity

flush(stderr()); flush(stdout())

### Name: normalize_intensity
### Title: Normalize intensity
### Aliases: normalize_intensity

### ** Examples

# A valid file properly populated
LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile)

# pmin = 15 because it is an extremely small file
# strongly decimated to reduce its size. There are
# actually few multiple returns
sensor <- track_sensor(las, Roussel2020(pmin = 15))

# Here the effect is virtually null because the size of
# the sample is too small to notice any effect of range
las <- normalize_intensity(las, range_correction(sensor, Rs = 2000))



cleanEx()
nameEx("p2r")
### * p2r

flush(stderr()); flush(stdout())

### Name: p2r
### Title: Digital Surface Model Algorithm
### Aliases: p2r

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile)
col <- height.colors(50)

# Points-to-raster algorithm with a resolution of 1 meter
chm <- grid_canopy(las, res = 1, p2r())
plot(chm, col = col)

# Points-to-raster algorithm with a resolution of 0.5 meters replacing each
# point by a 20 cm radius circle of 8 points
chm <- grid_canopy(las, res = 0.5, p2r(0.2))
plot(chm, col = col)

## Not run: 
##D chm <- grid_canopy(las, res = 0.5, p2r(0.2, na.fill = tin()))
##D plot(chm, col = col)
## End(Not run)



cleanEx()
nameEx("pitfree")
### * pitfree

flush(stderr()); flush(stdout())

### Name: pitfree
### Title: Digital Surface Model Algorithm
### Aliases: pitfree

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
poi = "-drop_z_below 0 -inside 481280 3812940 481330 3812990"
las <- readLAS(LASfile, filter = poi)
col <- height.colors(50)

# Basic triangulation and rasterization of first returns
chm <- grid_canopy(las, res = 0.5, dsmtin())
plot(chm, col = col)

# Khosravipour et al. pitfree algorithm
chm <- grid_canopy(las, res = 0.5, pitfree(c(0,2,5,10,15), c(0, 1.5)))
plot(chm, col = col)

## Not run: 
##D # Potentially complex concave subset of point cloud
##D x = c(481340, 481340, 481280, 481300, 481280, 481340)
##D y = c(3812940, 3813000, 3813000, 3812960, 3812940, 3812940)
##D las2 = clip_polygon(las,x,y)
##D plot(las2)
##D 
##D # Since the TIN interpolation is done within the convex hull of the point cloud
##D # dummy pixels are interpolated that are strictly correct according to the interpolation method
##D # used, but meaningless in our CHM
##D chm <- grid_canopy(las2, res = 0.5, pitfree())
##D plot(chm, col = col)
##D 
##D chm = grid_canopy(las2, res = 0.5, pitfree(max_edge = c(3, 1.5)))
##D plot(chm, col = col)
## End(Not run)



cleanEx()
nameEx("plot")
### * plot

flush(stderr()); flush(stdout())

### Name: plot
### Title: Plot a LAS* object
### Aliases: plot plot,LAS,missing-method plot,LAScatalog,missing-method
###   plot,LASheader,missing-method

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile)

plot(las)
plot(las, color = "Intensity")

## Not run: 
##D # If outliers break the color range, use the trim parameter
##D plot(las, color = "Intensity", trim = 150)
##D 
##D plot(las, color = "Classification")
##D 
##D # This dataset is already tree segmented
##D plot(las, color = "treeID")
## End(Not run)

# single file LAScatalog using data provided in lidR
ctg = readLAScatalog(LASfile)
plot(ctg)




cleanEx()
nameEx("plot.lasmetrics3d")
### * plot.lasmetrics3d

flush(stderr()); flush(stdout())

### Name: plot.lasmetrics3d
### Title: Plot voxelized LiDAR data
### Aliases: plot.lasmetrics3d

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
lidar = readLAS(LASfile)

voxels = voxel_metrics(lidar, list(Imean = mean(Intensity)), res = 5)
plot(voxels, color = "Imean", colorPalette = heat.colors(50), trim=60)



cleanEx()
nameEx("plot_3d")
### * plot_3d

flush(stderr()); flush(stdout())

### Name: plot_3d
### Title: Add a spatial object to a point cloud scene
### Aliases: plot_3d plot_dtm3d add_dtm3d add_treetops3d add_flightlines3d

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile, filter = "-keep_xy 273450 273600 5274450 5274600")

dtm = grid_terrain(las, algorithm = tin())
ttops <- find_trees(las, lmf(ws = 5))

plot_dtm3d(dtm)

x = plot(las)
add_dtm3d(x, dtm)
add_treetops3d(x, ttops)

## Not run: 
##D library(magrittr)
##D plot(las) %>% add_dtm3d(dtm) %>% add_treetops3d(ttops)
## End(Not run)



cleanEx()
nameEx("pmf")
### * pmf

flush(stderr()); flush(stdout())

### Name: pmf
### Title: Ground Segmentation Algorithm
### Aliases: pmf

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, select = "xyzrn", filter = "-inside 273450 5274350 273550 5274450")

ws <- seq(3,12, 3)
th <- seq(0.1, 1.5, length.out = length(ws))

las <- classify_ground(las, pmf(ws, th))
#plot(las, color = "Classification")



cleanEx()
nameEx("point_metrics")
### * point_metrics

flush(stderr()); flush(stdout())

### Name: point_metrics
### Title: Point-based metrics
### Aliases: point_metrics

### ** Examples

## Not run: 
##D LASfile <- system.file("extdata", "Topography.laz", package="lidR")
##D 
##D # Read only 0.5 points/m^2 for the purposes of this example
##D las = readLAS(LASfile, filter = "-thin_with_grid 2")
##D 
##D # Computes the eigenvalues of the covariance matrix of the neighbouring
##D # points and applies a test on these values. This function simulates the
##D # 'shp_plane()' algorithm from 'segment_shape()'
##D plane_metrics1 = function(x,y,z, th1 = 25, th2 = 6) {
##D   xyz <- cbind(x,y,z)
##D   cov_m <- cov(xyz)
##D   eigen_m <- eigen(cov_m)$value
##D   is_planar <- eigen_m[2] > (th1*eigen_m[3]) && (th2*eigen_m[2]) > eigen_m[1]
##D   return(list(planar = is_planar))
##D }
##D 
##D # Apply a user-defined function
##D M <- point_metrics(las, ~plane_metrics1(X,Y,Z), k = 25)
##D #> Computed in 6.3 seconds
##D 
##D # We can verify that it returns the same as 'shp_plane'
##D las <- segment_shape(las, shp_plane(k = 25), "planar")
##D #> Computed in 0.1 seconds
##D 
##D all.equal(M$planar, las$planar)
##D 
##D # At this stage we can be clever and find that the bottleneck is
##D # the eigenvalue computation. Let's write a C++ version of it with
##D # Rcpp and RcppArmadillo
##D Rcpp::sourceCpp(code = "
##D #include <RcppArmadillo.h>
##D // [[Rcpp::depends(RcppArmadillo)]]
##D 
##D // [[Rcpp::export]]
##D SEXP eigen_values(arma::mat A) {
##D arma::mat coeff;
##D arma::mat score;
##D arma::vec latent;
##D arma::princomp(coeff, score, latent, A);
##D return(Rcpp::wrap(latent));
##D }")
##D 
##D plane_metrics2 = function(x,y,z, th1 = 25, th2 = 6) {
##D   xyz <- cbind(x,y,z)
##D   eigen_m <- eigen_values(xyz)
##D   is_planar <- eigen_m[2] > (th1*eigen_m[3]) && (th2*eigen_m[2]) > eigen_m[1]
##D   return(list(planar = is_planar))
##D }
##D 
##D M <- point_metrics(las, ~plane_metrics2(X,Y,Z), k = 25)
##D #> Computed in 0.5 seconds
##D 
##D all.equal(M$planar, las$planar)
##D # Here we can see that the optimized version is way better but is still 5-times slower
##D # because of the overhead of calling R functions and switching back and forth from R to C++.
##D 
##D 
##D # Use the filter argument to process only first returns
##D M1 <- point_metrics(las, ~plane_metrics2(X,Y,Z), k = 25, filter = ~ReturnNumber == 1)
##D dim(M1) # 13894 instead of 17182 previously.
##D 
##D # is a memory-optimized equivalent to:
##D first = filter_first(las)
##D M2 <- point_metrics(first, ~plane_metrics2(X,Y,Z), k = 25)
##D all.equal(M1, M2)
## End(Not run)



cleanEx()
nameEx("projection")
### * projection

flush(stderr()); flush(stdout())

### Name: projection
### Title: Get or set the projection of a LAS* object
### Aliases: projection epsg epsg<- epsg,LASheader-method
###   epsg<-,LASheader-method epsg,LAS-method epsg<-,LAS-method wkt<-
###   wkt,LASheader-method wkt<-,LASheader-method wkt,LAS-method
###   wkt<-,LAS-method projection,LASheader-method projection,LAS-method
###   projection<-,LAS-method projection,LAScatalog-method
###   crs,LASheader-method crs,LAS-method crs<-,LAS-method
###   crs,LAScatalog-method

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile)

# Get the EPSG code stored in the header (returns 0 if not recorded)
epsg(las)

# Get the WKT string stored in the header (LAS >= 1.4)
wkt(las)

if (rgdal::new_proj_and_gdal())
{
   # Get the WKT of the CRS
   sp::wkt(crs(las))

   # Recorded CRS is "NAD83 / UTM zone 17N"
   sf::st_crs(las)$input
}

# Overwrite the CRS (but does not reproject)
crs <- sp::CRS("+init=epsg:26918")
projection(las) <- crs
sf::st_crs(las)$input

# Uses the EPSG code
projection(las) <- 26919
sf::st_crs(las)$input

# Uses a crs from sf
crs <- sf::st_crs(3035)
projection(las) <- crs




cleanEx()
nameEx("random")
### * random

flush(stderr()); flush(stdout())

### Name: random
### Title: Point Cloud Decimation Algorithm
### Aliases: random

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile, select = "xyz")

# Reach a pulse density of 1 on the overall dataset
thinned1 = decimate_points(las, random(1))
plot(grid_density(las))
plot(grid_density(thinned1))



cleanEx()
nameEx("range_correction")
### * range_correction

flush(stderr()); flush(stdout())

### Name: range_correction
### Title: Intensity normalization algorithm
### Aliases: range_correction get_range

### ** Examples

# A valid file properly populated
LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile)

# pmin = 15 because it is an extremely tiny file
# strongly decimated to reduce its size. There are
# actually few multiple returns
sensor <- track_sensor(las, Roussel2020(pmin = 15))

# Here the effect is virtually null because the size of
# the sample is too small to notice any effect of range
las <- normalize_intensity(las, range_correction(sensor, Rs = 2000))

# This might be useful for some applications
R = get_range(las, sensor)



cleanEx()
nameEx("readLAS")
### * readLAS

flush(stderr()); flush(stdout())

### Name: readLAS
### Title: Read .las or .laz files
### Aliases: readLAS readALSLAS readTLSLAS readUAVLAS readDAPLAS readMSLAS

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile)
las = readLAS(LASfile, select = "xyz")
las = readLAS(LASfile, select = "xyzi", filter = "-keep_first")
las = readLAS(LASfile, select = "xyziar", filter = "-keep_first -drop_z_below 0")

# Negation of attributes is also possible (all except intensity and angle)
las = readLAS(LASfile, select = "* -i -a")



cleanEx()
nameEx("readLAScatalog")
### * readLAScatalog

flush(stderr()); flush(stdout())

### Name: readLAScatalog
### Title: Create an object of class LAScatalog
### Aliases: readLAScatalog readALSLAScatalog readTLSLAScatalog
###   readUAVLAScatalog readDAPLAScatalog catalog

### ** Examples

# A single file LAScatalog using data provided with the package
LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
ctg = readLAScatalog(LASfile)
plot(ctg)

## Not run: 
##D ctg <- readLAScatalog("/path/to/a/folder/of/las/files")
##D 
##D # Internal engine will sequentially process chunks of size 500 x 500 m
##D opt_chunk_size(ctg) <- 500
##D 
##D # Internal engine will align the 500 x 500 m chunks on x = 250 and y = 300
##D opt_alignment(ctg) <- c(250, 300)
##D 
##D # Internal engine will not display a progress estimation
##D opt_progress(ctg) <- FALSE
##D 
##D # Internal engine will not return results into R. Instead it will write results in files.
##D opt_output_files(ctg) <- "/path/to/folder/templated_filename_{XBOTTOM}_{ID}"
##D 
##D # More details in the documentation
##D help("LAScatalog-class", "lidR")
##D help("catalog_options_tools", "lidR")
## End(Not run)



cleanEx()
nameEx("readLASheader")
### * readLASheader

flush(stderr()); flush(stdout())

### Name: readLASheader
### Title: Read a .las or .laz file header
### Aliases: readLASheader

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
header = readLASheader(LASfile)

print(header)
plot(header)

## Not run: 
##D plot(header, mapview = TRUE)
## End(Not run)



cleanEx()
nameEx("redefined_behaviors")
### * redefined_behaviors

flush(stderr()); flush(stdout())

### Name: $<-,LAS-method
### Title: Inherited but modified methods from sp
### Aliases: $<-,LAS-method [[<-,LAS,ANY,missing-method
###   [,LAS,numeric,ANY-method [,LAScatalog,ANY,ANY-method
###   [[<-,LAScatalog,ANY,ANY-method $<-,LAScatalog-method

### ** Examples

## Not run: 
##D LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
##D las = readLAS(LASfile)
##D 
##D las$Z = 2L
##D las[["Z"]] = 1:10
##D las$NewCol = 0
##D las[["NewCol"]] = 0
## End(Not run)



cleanEx()
nameEx("rumple_index")
### * rumple_index

flush(stderr()); flush(stdout())

### Name: rumple_index
### Title: Rumple index of roughness
### Aliases: rumple_index

### ** Examples

x <- runif(20, 0, 100)
y <- runif(20, 0, 100)

# Perfectly flat surface, rumple_index = 1
z <- rep(10, 20)
rumple_index(x, y, z)

# Rough surface, rumple_index > 1
z <- runif(20, 0, 10)
rumple_index(x, y, z)

# Rougher surface, rumple_index increases
z <- runif(20, 0, 50)
rumple_index(x, y, z)

# Measure of roughness is scale-dependent
rumple_index(x, y, z)
rumple_index(x/10, y/10, z)

# Use with a canopy height model
LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile)
chm <- grid_canopy(las, 2, p2r())
rumple_index(chm)



cleanEx()
nameEx("segment_shapes")
### * segment_shapes

flush(stderr()); flush(stdout())

### Name: segment_shapes
### Title: Estimation of the shape of the points neighborhood
### Aliases: segment_shapes

### ** Examples

## Not run: 
##D LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
##D las <- readLAS(LASfile)
##D 
##D las <- segment_shapes(las, shp_plane(k = 15), "Coplanar")
##D #plot(las, color = "Coplanar")
##D 
##D # Drop ground point at runtime
##D las <- segment_shapes(las, shp_plane(k = 15), "Coplanar", filter = ~Classification != 2L)
##D #plot(las, color = "Coplanar")
## End(Not run)




cleanEx()
nameEx("segment_snags")
### * segment_snags

flush(stderr()); flush(stdout())

### Name: segment_snags
### Title: Snag classification
### Aliases: segment_snags

### ** Examples

## Not run: 
##D LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
##D las <- readLAS(LASfile, select = "xyzi", filter="-keep_first") # Wing also included -keep_single
##D 
##D # For the Wing2015 method, supply a matrix of snag BranchBolePtRatio conditional
##D # assessment thresholds (see Wing et al. 2015, Table 2, pg. 172)
##D bbpr_thresholds <- matrix(c(0.80, 0.80, 0.70,
##D                           0.85, 0.85, 0.60,
##D                           0.80, 0.80, 0.60,
##D                           0.90, 0.90, 0.55),
##D                           nrow =3, ncol = 4)
##D 
##D # Run snag classification and assign classes to each point
##D las <- segment_snags(las, wing2015(neigh_radii = c(1.5, 1, 2), BBPRthrsh_mat = bbpr_thresholds))
##D 
##D # Plot it all, tree and snag points...
##D plot(las, color="snagCls", colorPalette = rainbow(5))
##D 
##D # Filter and plot snag points only
##D snags <- filter_poi(las, snagCls > 0)
##D plot(snags, color="snagCls", colorPalette = rainbow(5)[-1])
##D 
##D # Wing et al's (2015) methods ended with performing tree segmentation on the
##D # classified and filtered point cloud using the watershed method
## End(Not run)



cleanEx()
nameEx("segment_trees")
### * segment_trees

flush(stderr()); flush(stdout())

### Name: segment_trees
### Title: Individual tree segmentation
### Aliases: segment_trees

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las <- readLAS(LASfile, select = "xyz", filter = "-drop_z_below 0")

# Using Li et al. (2012)
las <- segment_trees(las, li2012(R = 3, speed_up = 5))
plot(las, color = "treeID")



cleanEx()
nameEx("silva2016")
### * silva2016

flush(stderr()); flush(stdout())

### Name: silva2016
### Title: Individual Tree Segmentation Algorithm
### Aliases: silva2016

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
poi <- "-drop_z_below 0 -inside 481280 3812940 481320 3812980"
las <- readLAS(LASfile, select = "xyz", filter = poi)
col <- pastel.colors(200)

chm <- grid_canopy(las, res = 0.5, p2r(0.3))
ker <- matrix(1,3,3)
chm <- raster::focal(chm, w = ker, fun = mean, na.rm = TRUE)

ttops <- find_trees(chm, lmf(4, 2))
las   <- segment_trees(las, silva2016(chm, ttops))
#plot(las, color = "treeID", colorPalette = col)



cleanEx()
nameEx("smooth_height")
### * smooth_height

flush(stderr()); flush(stdout())

### Name: smooth_height
### Title: Smooth a point cloud
### Aliases: smooth_height unsmooth_height

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile, select = "xyz")

las <- filter_surfacepoints(las, 1)
#plot(las)

las <- smooth_height(las, 5, "gaussian", "circle", sigma = 2)
#plot(las)

las <- unsmooth_height(las)
#plot(las)



cleanEx()
nameEx("sor")
### * sor

flush(stderr()); flush(stdout())

### Name: sor
### Title: Noise Segmentation Algorithm
### Aliases: sor

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las <- readLAS(LASfile, filter = "-inside 273450 5274350 273550 5274450")

# Add some artificial outliers because the original
# dataset is 'clean'
set.seed(314)
id = round(runif(20, 0, npoints(las)))
set.seed(42)
err = runif(20, -50, 50)
las$Z[id] = las$Z[id] + err

las <- classify_noise(las, sor(15,7))



cleanEx()
nameEx("stdmetrics")
### * stdmetrics

flush(stderr()); flush(stdout())

### Name: stdmetrics
### Title: Predefined standard metrics functions
### Aliases: stdmetrics stdmetrics_z stdmetrics_i stdmetrics_rn
###   stdmetrics_pulse stdmetrics_ctrl stdtreemetrics stdshapemetrics
###   .stdmetrics .stdmetrics_z .stdmetrics_i .stdmetrics_rn
###   .stdmetrics_pulse .stdmetrics_ctrl .stdtreemetrics .stdshapemetrics
### Keywords: datasets

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile, select = "*", filter = "-keep_random_fraction 0.5")

# All the predefined metrics
m1 <- grid_metrics(las, ~stdmetrics(X,Y,Z,Intensity,ReturnNumber,Classification,dz=1), res = 40)

# Convenient shortcut
m2 <- grid_metrics(las, .stdmetrics, res = 40)

# Basic metrics from intensities
m3 <- grid_metrics(las, ~stdmetrics_i(Intensity), res = 40)

# All the metrics from intensities
m4 <- grid_metrics(las, ~stdmetrics_i(Intensity, Z, Classification, ReturnNumber), res = 40)

# Convenient shortcut for the previous example
m5 <- grid_metrics(las, .stdmetrics_i, res = 40)

# Works also with cloud_metrics and hexbin_metrics
m6 <- cloud_metrics(las, .stdmetrics)
m7 <- hexbin_metrics(las, .stdmetrics)

# Combine some predefined function with your own new metrics
# Here convenient shortcuts are no longer usable.
myMetrics = function(z, i, rn)
{
  first  <- rn == 1L
  zfirst <- z[first]
  nfirst <- length(zfirst)
  above2 <- sum(z > 2)

  x <- above2/nfirst*100

  # User's metrics
  metrics <- list(
     above2aboven1st = x,       # Num of returns above 2 divided by num of 1st returns
     zimean  = mean(z*i),       # Mean products of z by intensity
     zsqmean = sqrt(mean(z^2))  # Quadratic mean of z
   )

  # Combined with standard metrics
  return( c(metrics, stdmetrics_z(z)) )
}

m10 <- grid_metrics(las, ~myMetrics(Z, Intensity, ReturnNumber), res = 40)

# Users can write their own convenient shorcuts like this:
.myMetrics = ~myMetrics(Z, Intensity, ReturnNumber)
m11 <- grid_metrics(las, .myMetrics, res = 40)




cleanEx()
nameEx("tin")
### * tin

flush(stderr()); flush(stdout())

### Name: tin
### Title: Spatial Interpolation Algorithm
### Aliases: tin

### ** Examples

LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile, filter = "-inside 273450 5274350 273550 5274450")

#plot(las)

dtm = grid_terrain(las, algorithm = tin())

plot(dtm, col = terrain.colors(50))
#plot_dtm3d(dtm)



cleanEx()
nameEx("track_sensor")
### * track_sensor

flush(stderr()); flush(stdout())

### Name: track_sensor
### Title: Reconstruct the trajectory of the LiDAR sensor using multiple
###   returns
### Aliases: track_sensor

### ** Examples

# A valid file properly populated
LASfile <- system.file("extdata", "Topography.laz", package="lidR")
las = readLAS(LASfile,
              select = "xyzrntp",
              filter = "-drop_single -thin_pulses_with_time 0.001")
#plot(las)

# pmin = 15 because it is an extremely small file
# strongly decimated to reduce its size. There are
# actually few multiple returns
flightlines <- track_sensor(las, Roussel2020(pmin = 15))

plot(las@header)
plot(flightlines, add = TRUE)

#x <- plot(las)
#add_flightlines3d(x, flightlines, radius = 10)

## Not run: 
##D # With a LAScatalog "-drop_single" and "-thin_pulses_with_time"
##D # are used by default
##D ctg = readLAScatalog("folder/")
##D flightlines <- track_sensor(ctg,  Roussel2020(pmin = 15))
##D plot(flightlines)
## End(Not run)



cleanEx()
nameEx("tree_metrics")
### * tree_metrics

flush(stderr()); flush(stdout())

### Name: tree_metrics
### Title: Compute metrics for each tree
### Aliases: tree_metrics

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
las = readLAS(LASfile, filter = "-drop_z_below 0")

# NOTE: This dataset is already segmented
# plot(las, color = "treeID", colorPalette = pastel.colors(200))

# Default computes only Z max
metrics = tree_metrics(las)

# User-defined metrics - mean height and mean intensity for each tree
metrics = tree_metrics(las, ~list(Zmean = mean(Z), Imean = mean(Intensity)))

# Define your own new metrics function
myMetrics = function(z, i)
{
  metrics = list(
     imean = mean(i),
     imax  = max(i),
     npoint = length(z)
   )

   return(metrics)
}

metrics = tree_metrics(las, ~myMetrics(Z, Intensity))

# predefined metrics (see ?stdmetrics)
metrics = tree_metrics(las, .stdtreemetrics)



cleanEx()
nameEx("util_makeZhangParam")
### * util_makeZhangParam

flush(stderr()); flush(stdout())

### Name: util_makeZhangParam
### Title: Parameters for progressive morphological filter
### Aliases: util_makeZhangParam

### ** Examples

p = util_makeZhangParam()



cleanEx()
nameEx("voxel_metrics")
### * voxel_metrics

flush(stderr()); flush(stdout())

### Name: voxel_metrics
### Title: Voxelize the space and compute metrics for each voxel
### Aliases: voxel_metrics

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las <- readLAS(LASfile)

# Cloud of points is voxelized with a 8-meter resolution and in each voxel
# the number of points is computed.
vm <- voxel_metrics(las, ~length(Z), 8)

# Cloud of points is voxelized with a 8-meter resolution and in each voxel
# the mean intensity of points is computed.
vm <- voxel_metrics(las, ~mean(Intensity), 8)
#plot(vm, color = "V1", colorPalette = heat.colors(50), trim = 60)

# Define your own metric function
myMetrics = function(i)
{
  ret = list(
     npoints = length(i),
     imean   = mean(i)
   )

   return(ret)
}

voxels <- voxel_metrics(las, ~myMetrics(Intensity), 8)

#plot(voxels, color = "imean", colorPalette = heat.colors(50), trim = 60)
#etc.

attr(voxels, "res")



cleanEx()
nameEx("voxelize_points")
### * voxelize_points

flush(stderr()); flush(stdout())

### Name: voxelize_points
### Title: Voxelize a point cloud
### Aliases: voxelize_points

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile, select = "xyz")

las2 = voxelize_points(las, 2)
#plot(las2)



cleanEx()
nameEx("watershed")
### * watershed

flush(stderr()); flush(stdout())

### Name: watershed
### Title: Individual Tree Segmentation Algorithm
### Aliases: watershed mcwatershed

### ** Examples




cleanEx()
nameEx("wing2015")
### * wing2015

flush(stderr()); flush(stdout())

### Name: wing2015
### Title: Snags Segmentation Algorithm
### Aliases: wing2015

### ** Examples

LASfile <- system.file("extdata", "MixedConifer.laz", package="lidR")
# Wing also included -keep_single
poi ="-keep_first -inside 481260 3812920 481310 3812960"
las <- readLAS(LASfile, select = "xyzi", filter = poi)

# For the Wing2015 method, supply a matrix of snag BranchBolePtRatio conditional
# assessment thresholds (see Wing et al. 2015, Table 2, pg. 172)
bbpr_thresholds <- matrix(c(0.80, 0.80, 0.70,
                          0.85, 0.85, 0.60,
                          0.80, 0.80, 0.60,
                          0.90, 0.90, 0.55),
                          nrow =3, ncol = 4)

# Run snag classification and assign classes to each point
las <- segment_snags(las, wing2015(neigh_radii = c(1.5, 1, 2), BBPRthrsh_mat = bbpr_thresholds))

# Plot it all, tree and snag points...
#plot(las, color="snagCls", colorPalette = rainbow(5))

# Filter and plot snag points only
snags <- filter_poi(las, snagCls > 0)
#plot(snags, color="snagCls", colorPalette = rainbow(5)[-1])

# Wing et al's (2015) methods ended with performing tree segmentation on the
# classified and filtered point cloud using the watershed method




cleanEx()
nameEx("writeLAS")
### * writeLAS

flush(stderr()); flush(stdout())

### Name: writeLAS
### Title: Write a .las or .laz file
### Aliases: writeLAS

### ** Examples

LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
las = readLAS(LASfile)
subset = clip_rectangle(las, 684850, 5017850, 684900, 5017900)
writeLAS(subset, tempfile(fileext = ".laz"))



### * <FOOTER>
###
cleanEx()
options(digits = 7L)
base::cat("Time elapsed: ", proc.time() - base::get("ptime", pos = 'CheckExEnv'),"\n")
grDevices::dev.off()
###
### Local variables: ***
### mode: outline-minor ***
### outline-regexp: "\\(> \\)?### [*]+" ***
### End: ***
quit('no')
